"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.I18n = void 0;
const bignumber_js_1 = __importDefault(require("bignumber.js"));
const lodash_1 = require("lodash");
const Locales_1 = require("./Locales");
const Pluralization_1 = require("./Pluralization");
const MissingTranslation_1 = require("./MissingTranslation");
const helpers_1 = require("./helpers");
const within = (start, end, actual) => actual >= start && actual <= end;
const DEFAULT_I18N_OPTIONS = {
    defaultLocale: "en",
    locale: "en",
    defaultSeparator: ".",
    placeholder: /(?:\{\{|%\{)(.*?)(?:\}\}?)/gm,
    enableFallback: false,
    translations: {},
    missingBehavior: "message",
    missingTranslationPrefix: "",
    missingPlaceholder: (_i18n, placeholder) => `[missing "${placeholder}" value]`,
    nullPlaceholder: (i18n, placeholder, message, options) => i18n.missingPlaceholder(i18n, placeholder, message, options),
    transformKey: (key) => key,
};
const STORAGE_UNITS = ["byte", "kb", "mb", "gb", "tb", "pb", "eb"];
const WORD_CONNECTORS = {
    wordsConnector: ", ",
    twoWordsConnector: " and ",
    lastWordConnector: ", and ",
};
const DECIMAL_UNITS = {
    "0": "unit",
    "1": "ten",
    "2": "hundred",
    "3": "thousand",
    "6": "million",
    "9": "billion",
    "12": "trillion",
    "15": "quadrillion",
    "-1": "deci",
    "-2": "centi",
    "-3": "mili",
    "-6": "micro",
    "-9": "nano",
    "-12": "pico",
    "-15": "femto",
};
const INVERTED_DECIMAL_UNITS = lodash_1.zipObject(Object.values(DECIMAL_UNITS), Object.keys(DECIMAL_UNITS).map((key) => parseInt(key, 10)));
class I18n {
    constructor(translations = {}, options) {
        this._locale = DEFAULT_I18N_OPTIONS.locale;
        this._defaultLocale = DEFAULT_I18N_OPTIONS.defaultLocale;
        this._version = 0;
        this.onChangeHandlers = [];
        this.translations = {};
        this.t = this.translate;
        this.p = this.pluralize;
        this.l = this.localize;
        this.distanceOfTimeInWords = this.timeAgoInWords;
        const { locale, enableFallback, missingBehavior, missingTranslationPrefix, missingPlaceholder, nullPlaceholder, defaultLocale, defaultSeparator, placeholder, transformKey, } = Object.assign(Object.assign({}, DEFAULT_I18N_OPTIONS), options);
        this.locale = locale;
        this.defaultLocale = defaultLocale;
        this.defaultSeparator = defaultSeparator;
        this.enableFallback = enableFallback;
        this.locale = locale;
        this.missingBehavior = missingBehavior;
        this.missingTranslationPrefix = missingTranslationPrefix;
        this.missingPlaceholder = missingPlaceholder;
        this.nullPlaceholder = nullPlaceholder;
        this.placeholder = placeholder;
        this.pluralization = new Pluralization_1.Pluralization(this);
        this.locales = new Locales_1.Locales(this);
        this.missingTranslation = new MissingTranslation_1.MissingTranslation(this);
        this.transformKey = transformKey;
        this.store(translations);
    }
    store(translations) {
        const map = helpers_1.propertyFlatList(translations);
        map.forEach((path) => lodash_1.set(this.translations, path, lodash_1.get(translations, path)));
        this.hasChanged();
    }
    get locale() {
        return this._locale || this.defaultLocale || "en";
    }
    set locale(newLocale) {
        if (typeof newLocale !== "string") {
            throw new Error(`Expected newLocale to be a string; got ${helpers_1.inferType(newLocale)}`);
        }
        const changed = this._locale !== newLocale;
        this._locale = newLocale;
        if (changed) {
            this.hasChanged();
        }
    }
    get defaultLocale() {
        return this._defaultLocale || "en";
    }
    set defaultLocale(newLocale) {
        if (typeof newLocale !== "string") {
            throw new Error(`Expected newLocale to be a string; got ${helpers_1.inferType(newLocale)}`);
        }
        const changed = this._defaultLocale !== newLocale;
        this._defaultLocale = newLocale;
        if (changed) {
            this.hasChanged();
        }
    }
    numberToCurrency(amount, options = {}) {
        options = Object.assign(Object.assign(Object.assign({ unit: "$", precision: 2, format: "%u%n", delimiter: ",", separator: "." }, this.get("number.format")), this.get("number.currency.format")), options);
        options.negativeFormat = options.negativeFormat || `-${options.format}`;
        return helpers_1.formatNumber(amount, options);
    }
    translate(scope, options) {
        options = Object.assign({}, options);
        const translationOptions = helpers_1.createTranslationOptions(this, scope, options);
        let translation;
        const hasFoundTranslation = translationOptions.some((translationOption) => {
            if (helpers_1.isSet(translationOption.scope)) {
                translation = helpers_1.lookup(this, translationOption.scope, options);
            }
            else if (helpers_1.isSet(translationOption.message)) {
                translation = translationOption.message;
            }
            return translation !== undefined && translation !== null;
        });
        if (!hasFoundTranslation) {
            return this.missingTranslation.get(scope, options);
        }
        if (typeof translation === "string") {
            translation = helpers_1.interpolate(this, translation, options);
        }
        else if (typeof translation === "object" &&
            translation &&
            helpers_1.isSet(options.count)) {
            translation = this.pluralize(options.count || 0, translation, options);
        }
        return translation;
    }
    pluralize(count, scope, options) {
        return helpers_1.pluralize(this, count, scope, Object.assign({}, options));
    }
    localize(type, value, options) {
        options = Object.assign({}, options);
        if (value === undefined || value === null) {
            return "";
        }
        switch (type) {
            case "currency":
                return this.numberToCurrency(value);
            case "number":
                return helpers_1.formatNumber(value, Object.assign({ delimiter: ",", precision: 3, separator: ".", significant: false, stripInsignificantZeros: false }, helpers_1.lookup(this, "number.format")));
            case "percentage":
                return this.numberToPercentage(value);
            default: {
                let localizedValue;
                if (type.match(/^(date|time)/)) {
                    localizedValue = this.toTime(type, value);
                }
                else {
                    localizedValue = value.toString();
                }
                return helpers_1.interpolate(this, localizedValue, options);
            }
        }
    }
    toTime(scope, input) {
        const date = helpers_1.parseDate(input);
        const format = helpers_1.lookup(this, scope);
        if (date.toString().match(/invalid/i)) {
            return date.toString();
        }
        if (!format) {
            return date.toString();
        }
        return this.strftime(date, format);
    }
    numberToPercentage(input, options = {}) {
        options = Object.assign(Object.assign({ unit: "%", precision: 3, separator: ".", delimiter: "", format: "%n%" }, helpers_1.camelCaseKeys(helpers_1.lookup(this, "number.percentage.format"))), options);
        return helpers_1.formatNumber(input, options);
    }
    numberToHumanSize(numeric, options = {}) {
        var _a, _b, _c, _d;
        options = Object.assign(Object.assign({ roundMode: "default", delimiter: "", precision: 3, significant: true, stripInsignificantZeros: true }, helpers_1.camelCaseKeys(this.get("number.human.format"))), options);
        const roundMode = helpers_1.expandRoundMode(options.roundMode || "default");
        const base = 1024;
        const num = new bignumber_js_1.default(numeric).abs();
        const smallerThanBase = num.lt(base);
        let numberToBeFormatted;
        const stripInsignificantZeros = (_a = options.stripInsignificantZeros) !== null && _a !== void 0 ? _a : true;
        const units = STORAGE_UNITS;
        const computeExponent = (numeric, units) => {
            const max = units.length - 1;
            const exp = new bignumber_js_1.default(Math.log(numeric.toNumber()))
                .div(Math.log(base))
                .integerValue(bignumber_js_1.default.ROUND_DOWN)
                .toNumber();
            return Math.min(max, exp);
        };
        const storageUnitKey = () => {
            const keyEnd = smallerThanBase ? "byte" : units[exponent];
            return `number.human.storage_units.units.${keyEnd}`;
        };
        const exponent = computeExponent(num, units);
        if (smallerThanBase) {
            numberToBeFormatted = num.integerValue();
        }
        else {
            numberToBeFormatted = new bignumber_js_1.default(helpers_1.roundNumber(num.div(Math.pow(base, exponent)), {
                significant: (_b = options.significant) !== null && _b !== void 0 ? _b : true,
                precision: (_c = options.precision) !== null && _c !== void 0 ? _c : 3,
                roundMode: (_d = options.roundMode) !== null && _d !== void 0 ? _d : "default",
            }));
        }
        let precision;
        if (helpers_1.inferType(options.precision) === "number") {
            precision = options.precision;
        }
        else {
            const significand = numberToBeFormatted.minus(numberToBeFormatted.integerValue());
            if (significand.gte(0.06)) {
                precision = 2;
            }
            else if (significand.gt(0)) {
                precision = 1;
            }
            else {
                precision = 0;
            }
        }
        const format = this.translate("number.human.storage_units.format", {
            defaultValue: "%n %u",
        });
        const unit = this.translate(storageUnitKey(), {
            count: num.integerValue().toNumber(),
        });
        let formattedNumber = numberToBeFormatted.toFixed(precision, roundMode);
        if (stripInsignificantZeros) {
            formattedNumber = formattedNumber
                .replace(/(\..*?)0+$/, "$1")
                .replace(/\.$/, "");
        }
        return format.replace("%n", formattedNumber).replace("%u", unit);
    }
    numberToHuman(input, options = {}) {
        var _a, _b, _c, _d, _e, _f;
        options = Object.assign(Object.assign({ delimiter: "", precision: 3, significant: true, stripInsignificantZeros: true, format: "%n %u", units: (_a = this.get("number.human.decimal_units")) === null || _a === void 0 ? void 0 : _a.units }, helpers_1.camelCaseKeys(this.get("number.human.format"))), options);
        const roundMode = (_b = options.roundMode) !== null && _b !== void 0 ? _b : "default";
        const precision = (_c = options.precision) !== null && _c !== void 0 ? _c : 3;
        const significant = (_d = options.significant) !== null && _d !== void 0 ? _d : true;
        const format = (_e = options.format) !== null && _e !== void 0 ? _e : "%n %u";
        const separator = (_f = options.separator) !== null && _f !== void 0 ? _f : ".";
        let units;
        if (helpers_1.inferType(options.units) === "string") {
            units = this.get(options.units);
        }
        else {
            units = options.units;
        }
        let formattedNumber = helpers_1.roundNumber(new bignumber_js_1.default(input), {
            roundMode,
            precision,
            significant,
        });
        const unitExponents = (units) => lodash_1.sortBy(Object.keys(units).map((name) => INVERTED_DECIMAL_UNITS[name]), (numeric) => numeric * -1);
        const calculateExponent = (num, units) => {
            const exponent = num.isZero()
                ? 0
                : Math.floor(Math.log10(num.abs().toNumber()));
            return unitExponents(units).find((exp) => exponent >= exp) || 0;
        };
        const determineUnit = (units, exponent) => {
            const expName = DECIMAL_UNITS[exponent.toString()];
            return units[expName] || "";
        };
        const exponent = calculateExponent(new bignumber_js_1.default(formattedNumber), units);
        const unit = determineUnit(units, exponent);
        formattedNumber = helpers_1.roundNumber(new bignumber_js_1.default(formattedNumber).div(Math.pow(10, exponent)), {
            roundMode,
            precision,
            significant,
        });
        if (options.stripInsignificantZeros) {
            let [whole, significand] = formattedNumber.split(".");
            significand = (significand || "").replace(/0+$/, "");
            formattedNumber = whole;
            if (significand) {
                formattedNumber += `${separator}${significand}`;
            }
        }
        return format
            .replace("%n", formattedNumber || "0")
            .replace("%u", unit)
            .trim();
    }
    numberToRounded(input, options) {
        options = Object.assign({ unit: "", precision: 3, significant: false, separator: ".", delimiter: "", stripInsignificantZeros: false }, options);
        return helpers_1.formatNumber(input, options);
    }
    numberToDelimited(input, options = {}) {
        options = Object.assign({ delimiterPattern: /(\d)(?=(\d\d\d)+(?!\d))/g, delimiter: ",", separator: "." }, options);
        const numeric = new bignumber_js_1.default(input);
        if (!numeric.isFinite()) {
            return input.toString();
        }
        const newOptions = options;
        if (!newOptions.delimiterPattern.global) {
            throw new Error(`options.delimiterPattern must be a global regular expression; received ${newOptions.delimiterPattern}`);
        }
        let [left, right] = numeric.toString().split(".");
        left = left.replace(newOptions.delimiterPattern, (digitToDelimiter) => `${digitToDelimiter}${newOptions.delimiter}`);
        return [left, right].filter(Boolean).join(newOptions.separator);
    }
    withLocale(locale, callback) {
        return __awaiter(this, void 0, void 0, function* () {
            const originalLocale = this.locale;
            try {
                this.locale = locale;
                yield callback();
            }
            finally {
                this.locale = originalLocale;
            }
        });
    }
    strftime(date, format) {
        const options = Object.assign(Object.assign({}, helpers_1.camelCaseKeys(helpers_1.lookup(this, "date"))), { meridian: {
                am: helpers_1.lookup(this, "time.am") || "AM",
                pm: helpers_1.lookup(this, "time.pm") || "PM",
            } });
        return helpers_1.strftime(date, format, options);
    }
    update(path, override, options = { strict: false }) {
        if (options.strict && !lodash_1.has(this.translations, path)) {
            throw new Error(`The path "${path}" is not currently defined`);
        }
        const currentNode = lodash_1.get(this.translations, path);
        const currentType = helpers_1.inferType(currentNode);
        const overrideType = helpers_1.inferType(override);
        if (options.strict && currentType !== overrideType) {
            throw new Error(`The current type for "${path}" is "${currentType}", but you're trying to override it with "${overrideType}"`);
        }
        let newNode;
        if (overrideType === "object") {
            newNode = Object.assign(Object.assign({}, currentNode), override);
        }
        else {
            newNode = override;
        }
        lodash_1.set(this.translations, path, newNode);
        this.hasChanged();
    }
    toSentence(items, options) {
        options = Object.assign(Object.assign(Object.assign({}, WORD_CONNECTORS), helpers_1.camelCaseKeys(helpers_1.lookup(this, "support.array"))), options);
        const size = items.length;
        switch (size) {
            case 0:
                return "";
            case 1:
                return `${items[0]}`;
            case 2:
                return items.join(options.twoWordsConnector);
            default:
                return [
                    items.slice(0, size - 1).join(options.wordsConnector),
                    options.lastWordConnector,
                    items[size - 1],
                ].join("");
        }
    }
    timeAgoInWords(fromTime, toTime, options = {}) {
        const scope = options.scope || "datetime.distance_in_words";
        const t = (name, count = 0) => this.t(name, { count, scope });
        fromTime = helpers_1.parseDate(fromTime);
        toTime = helpers_1.parseDate(toTime);
        let fromInSeconds = fromTime.getTime() / 1000;
        let toInSeconds = toTime.getTime() / 1000;
        if (fromInSeconds > toInSeconds) {
            [fromTime, toTime, fromInSeconds, toInSeconds] = [
                toTime,
                fromTime,
                toInSeconds,
                fromInSeconds,
            ];
        }
        const distanceInSeconds = Math.round(toInSeconds - fromInSeconds);
        const distanceInMinutes = Math.round((toInSeconds - fromInSeconds) / 60);
        const distanceInHours = distanceInMinutes / 60;
        const distanceInDays = distanceInHours / 24;
        const distanceInHoursRounded = Math.round(distanceInMinutes / 60);
        const distanceInDaysRounded = Math.round(distanceInDays);
        const distanceInMonthsRounded = Math.round(distanceInDaysRounded / 30);
        if (within(0, 1, distanceInMinutes)) {
            if (!options.includeSeconds) {
                return distanceInMinutes === 0
                    ? t("less_than_x_minutes", 1)
                    : t("x_minutes", distanceInMinutes);
            }
            if (within(0, 4, distanceInSeconds)) {
                return t("less_than_x_seconds", 5);
            }
            if (within(5, 9, distanceInSeconds)) {
                return t("less_than_x_seconds", 10);
            }
            if (within(10, 19, distanceInSeconds)) {
                return t("less_than_x_seconds", 20);
            }
            if (within(20, 39, distanceInSeconds)) {
                return t("half_a_minute");
            }
            if (within(40, 59, distanceInSeconds)) {
                return t("less_than_x_minutes", 1);
            }
            return t("x_minutes", 1);
        }
        if (within(2, 44, distanceInMinutes)) {
            return t("x_minutes", distanceInMinutes);
        }
        if (within(45, 89, distanceInMinutes)) {
            return t("about_x_hours", 1);
        }
        if (within(90, 1439, distanceInMinutes)) {
            return t("about_x_hours", distanceInHoursRounded);
        }
        if (within(1440, 2519, distanceInMinutes)) {
            return t("x_days", 1);
        }
        if (within(2520, 43199, distanceInMinutes)) {
            return t("x_days", distanceInDaysRounded);
        }
        if (within(43200, 86399, distanceInMinutes)) {
            return t("about_x_months", Math.round(distanceInMinutes / 43200));
        }
        if (within(86400, 525599, distanceInMinutes)) {
            return t("x_months", distanceInMonthsRounded);
        }
        let fromYear = fromTime.getFullYear();
        if (fromTime.getMonth() + 1 >= 3) {
            fromYear += 1;
        }
        let toYear = toTime.getFullYear();
        if (toTime.getMonth() + 1 < 3) {
            toYear -= 1;
        }
        const leapYears = fromYear > toYear
            ? 0
            : lodash_1.range(fromYear, toYear).filter((year) => new Date(year, 1, 29).getMonth() == 1).length;
        const minutesInYear = 525600;
        const minuteOffsetForLeapYear = leapYears * 1440;
        const minutesWithOffset = distanceInMinutes - minuteOffsetForLeapYear;
        const distanceInYears = Math.trunc(minutesWithOffset / minutesInYear);
        const diff = parseFloat((minutesWithOffset / minutesInYear - distanceInYears).toPrecision(3));
        if (diff < 0.25) {
            return t("about_x_years", distanceInYears);
        }
        if (diff < 0.75) {
            return t("over_x_years", distanceInYears);
        }
        return t("almost_x_years", distanceInYears + 1);
    }
    onChange(callback) {
        this.onChangeHandlers.push(callback);
    }
    get version() {
        return this._version;
    }
    get(path) {
        return helpers_1.lookup(this, path);
    }
    runCallbacks() {
        this.onChangeHandlers.forEach((callback) => callback(this));
    }
    hasChanged() {
        this._version += 1;
        this.runCallbacks();
    }
}
exports.I18n = I18n;
//# sourceMappingURL=I18n.js.map